/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kdeconnect.daemon.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef KDECONNECT_DAEMON_INTERFACE_H
#define KDECONNECT_DAEMON_INTERFACE_H

#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.daemon
 */
class OrgKdeKdeconnectDaemonInterface : public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    {
        return "org.kde.kdeconnect.daemon";
    }

public:
    OrgKdeKdeconnectDaemonInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDaemonInterface();

    Q_PROPERTY(QStringList customDevices READ customDevices WRITE setCustomDevices)
    inline QStringList customDevices() const
    {
        return qvariant_cast<QStringList>(property("customDevices"));
    }
    inline void setCustomDevices(const QStringList &value)
    {
        setProperty("customDevices", QVariant::fromValue(value));
    }

    Q_PROPERTY(QStringList pairingRequests READ pairingRequests)
    inline QStringList pairingRequests() const
    {
        return qvariant_cast<QStringList>(property("pairingRequests"));
    }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QString> announcedName()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("announcedName"), argumentList);
    }

    inline QDBusPendingReply<QString> deviceIdByName(const QString &name)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(name);
        return asyncCallWithArgumentList(QStringLiteral("deviceIdByName"), argumentList);
    }

    inline QDBusPendingReply<QMap<QString, QString>> deviceNames()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("deviceNames"), argumentList);
    }

    inline QDBusPendingReply<QMap<QString, QString>> deviceNames(bool onlyReachable)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(onlyReachable);
        return asyncCallWithArgumentList(QStringLiteral("deviceNames"), argumentList);
    }

    inline QDBusPendingReply<QMap<QString, QString>> deviceNames(bool onlyReachable, bool onlyPaired)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(onlyReachable) << QVariant::fromValue(onlyPaired);
        return asyncCallWithArgumentList(QStringLiteral("deviceNames"), argumentList);
    }

    inline QDBusPendingReply<QStringList> devices()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("devices"), argumentList);
    }

    inline QDBusPendingReply<QStringList> devices(bool onlyReachable)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(onlyReachable);
        return asyncCallWithArgumentList(QStringLiteral("devices"), argumentList);
    }

    inline QDBusPendingReply<QStringList> devices(bool onlyReachable, bool onlyPaired)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(onlyReachable) << QVariant::fromValue(onlyPaired);
        return asyncCallWithArgumentList(QStringLiteral("devices"), argumentList);
    }

    inline QDBusPendingReply<> forceOnNetworkChange()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("forceOnNetworkChange"), argumentList);
    }

    inline QDBusPendingReply<QStringList> linkProviders()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("linkProviders"), argumentList);
    }

    inline QDBusPendingReply<QString> selfId()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("selfId"), argumentList);
    }

    inline QDBusPendingReply<> sendSimpleNotification(const QString &eventId, const QString &title, const QString &text, const QString &iconName)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(eventId) << QVariant::fromValue(title) << QVariant::fromValue(text) << QVariant::fromValue(iconName);
        return asyncCallWithArgumentList(QStringLiteral("sendSimpleNotification"), argumentList);
    }

    inline QDBusPendingReply<> setAnnouncedName(const QString &name)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(name);
        return asyncCallWithArgumentList(QStringLiteral("setAnnouncedName"), argumentList);
    }

    inline QDBusPendingReply<> setLinkProviderState(const QString &linkProvider, bool enabled)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(linkProvider) << QVariant::fromValue(enabled);
        return asyncCallWithArgumentList(QStringLiteral("setLinkProviderState"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void announcedNameChanged(const QString &announcedName);
    void customDevicesChanged(const QStringList &customDevices);
    void deviceAdded(const QString &id);
    void deviceListChanged();
    void deviceRemoved(const QString &id);
    void deviceVisibilityChanged(const QString &id, bool isVisible);
    void linkProvidersChanged(const QStringList &linkProviders);
    void pairingRequestsChanged();
};

namespace org
{
namespace kde
{
namespace kdeconnect
{
using daemon = ::OrgKdeKdeconnectDaemonInterface;
}
}
}

#endif
