/*
    SPDX-FileCopyrightText: 2010 Rafael Fernández López <ereslibre@kde.org>

    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#ifndef SOLID_BACKENDS_UDEV_H
#define SOLID_BACKENDS_UDEV_H

#include "../shared/udevqt.h"

#define UDEV_UDI_PREFIX "/org/kde/solid/udev"

#endif // SOLID_BACKENDS_UDEV_H
