/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kdeconnect.daemon.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "kdeconnect_daemon_interface.h"

/*
 * Implementation of interface class OrgKdeKdeconnectDaemonInterface
 */

OrgKdeKdeconnectDaemonInterface::OrgKdeKdeconnectDaemonInterface(const QString &service,
                                                                 const QString &path,
                                                                 const QDBusConnection &connection,
                                                                 QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgKdeKdeconnectDaemonInterface::~OrgKdeKdeconnectDaemonInterface()
{
}
