/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kdeconnect.device.battery.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef KDECONNECT_DEVICE_BATTERY_INTERFACE_H
#define KDECONNECT_DEVICE_BATTERY_INTERFACE_H

#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.battery
 */
class OrgKdeKdeconnectDeviceBatteryInterface : public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    {
        return "org.kde.kdeconnect.device.battery";
    }

public:
    OrgKdeKdeconnectDeviceBatteryInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceBatteryInterface();

    Q_PROPERTY(int charge READ charge)
    inline int charge() const
    {
        return qvariant_cast<int>(property("charge"));
    }

    Q_PROPERTY(bool isCharging READ isCharging)
    inline bool isCharging() const
    {
        return qvariant_cast<bool>(property("isCharging"));
    }

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
    void refreshed(bool isCharging, int charge);
};

#endif
